'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Log = exports.defaultOptions = exports.indent = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.formatError = formatError;
exports.fromEnv = fromEnv;

var _chalk = require('chalk');

var _dateformat = require('dateformat');

var _dateformat2 = _interopRequireDefault(_dateformat);

var _lodash = require('lodash.defaults');

var _lodash2 = _interopRequireDefault(_lodash);

var _jestMessageUtil = require('jest-message-util');

var _path = require('path');

var _purdy = require('purdy');

var _stripAnsi = require('strip-ansi');

var _stripAnsi2 = _interopRequireDefault(_stripAnsi);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Function that returns a Stream.
 *
 * @typedef {Function} Log~streamFn
 * @param {number} level - Level number.
 * @returns {Stream} A Stream
 */

/**
 * Options for `Log`.
 *
 * @typedef {Object} Log~options
 * @property {string[]} levels - Log levels.
 * @property {function[]} colors - Color function for each level.
 * @property {boolean[]} inverse - Whether to inverse color for each level.
 * @property {string[]} labels - Label of each level.
 * @property {string} separator - Separator between time and label, and between header and body
 * @property {Stream|Log~streamFn} stream - Stream or function that returns a Stream.
 * @property {boolean} displayTime - Whether to display time.
 * @property {string} displayTimeFormat - Format to display time in. See [dateformat].
 * @property {string} level - All log calls below this level will be ignored.
 *
 * [dateformat]: https://www.npmjs.com/package/dateformat
 */

/**
 * Add characters at the begenning of every line of a multi-line string.
 *
 * @param {string} string - Input string.
 * @param {number} length - Number of characters to insert at every line.
 * @param {string} [char=' '] - Character to insert.
 * @returns {string} The indented string.
 */
var indent = exports.indent = function indent(string, length) {
  var char = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : ' ';
  return '' + string.replace(/\n/g, '\n' + char.repeat(length));
};

/**
 * Transform an error into a user-readable, colored string.
 *
 * @param  {Error} error - Input error.
 * @returns {string} The formatted message.
 */
function formatError(error) {
  var _separateMessageFromS = (0, _jestMessageUtil.separateMessageFromStack)(error.stack),
      message = _separateMessageFromS.message,
      stack = _separateMessageFromS.stack;

  var stackTrace = (0, _jestMessageUtil.formatStackTrace)(stack, { rootDir: (0, _path.dirname)((0, _path.dirname)(__dirname)), testMatch: [] }, { noStackTrace: false });
  return (0, _chalk.red)(message) + '\n' + stackTrace.replace(/\n {4}/g, '\n') + '\n';
}

/**
 * Transform an object into a user-readable, colored string. If the resulting
 * string is shorter than 200 characters, inlines it.
 *
 * @param {Object} object - The input object.
 * @returns {string} A stringified version of the object.
 */
function formatObject(object) {
  var str = (0, _purdy.stringify)(object, { indent: 2, arrayIndex: false });
  return (0, _chalk.grey)((0, _stripAnsi2.default)(str).length < 200 ? str.replace(/\n */g, ' ') : '\n' + str + '\n');
}

/**
 * Transform a value of any type into a user-readable, colored string.
 *
 * @param {*} value - A value of any type.
 * @returns {string} A stringified version of the value.
 */
function format(value) {
  if (value instanceof Error) {
    return formatError(value);
  }
  return typeof value === 'string' ? value : formatObject(value);
}

/**
 * Default configuration
 * @type {Log~options}
 */
var defaultOptions = exports.defaultOptions = {
  levels: ['emergency', 'alert', 'critical', 'error', 'warning', 'notice', 'info', 'debug'],
  colors: [_chalk.red.bold, _chalk.red, _chalk.red.bold, _chalk.red, _chalk.magenta, _chalk.yellow, _chalk.blue, _chalk.grey],
  inverse: [true, true],
  labels: ['EMERG', 'ALERT', 'CRITI', 'ERROR', ' WARN', ' NOTE', ' INFO', 'DEBUG'],
  separator: ' • ',
  stream: function stream(level) {
    return level <= 4 ? process.stderr : process.stdout;
  },
  format: format,
  displayTime: false,
  displayTimeFormat: 'yyyy-mm-dd HH:MM:ss.l',
  level: 'info'
};

/**
 * If an option is a function, apply it with given parameters, otherwise returns it.
 *
 * @param {Function|*} option - The option bvalue or function.
 * @param {*[]} args - Arguments to call `option` with, if `option` is a function.
 * @returns {*} The return of `option(...args)`, or `option`.
 */
var applyOption = function applyOption(option) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  return typeof option === 'function' ? option.apply(undefined, args) : option;
};

/**
 * Fancy lightweight logging utility.
 *
 * @class

 */

var Log = exports.Log = function () {
  /**
   * Creates an instance of `Log`.
   *
   * @param {Log~options} [options={}] - Options.
   */
  function Log() {
    var _this = this;

    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Log);

    /**
     * @var {Log~options} options - Logger options.
     */
    this.options = (0, _lodash2.default)({}, options, defaultOptions);

    this.options.levels.forEach(function (level, logLevel) {
      /**
       * Logging method for each level of `options.levels`.
       *
       * @param {...*} parts - Any value: `string`, `Object`, `Error`, etc.
       * @returns {undefined} Nothing.
       */
      _this[level] = function () {
        for (var _len2 = arguments.length, parts = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          parts[_key2] = arguments[_key2];
        }

        return _this.write(logLevel, parts);
      };
    });
  }

  /**
   * @property {number} level - Integer value of `this.options.level`.
   */


  _createClass(Log, [{
    key: 'buildHeader',


    /**
     * Build the header.
     *
     * @param {number} level - Level number.
     * @returns {string} The constructed header.
     */
    value: function buildHeader(level) {
      var _options = this.options,
          labels = _options.labels,
          colors = _options.colors,
          inverse = _options.inverse,
          displayTime = _options.displayTime,
          displayTimeFormat = _options.displayTimeFormat,
          separator = _options.separator;

      var time = displayTime ? (0, _chalk.grey)((0, _dateformat2.default)(Date.now(), displayTimeFormat)) : '';
      var color = colors[level];
      var labelColor = inverse[level] ? color.inverse : color;
      var label = '' + labels[level];
      return '' + (time && (0, _chalk.grey)('' + time + separator)) + labelColor(label) + color(separator);
    }

    /**
     * Build the body.
     *
     * @param {*[]} parts - Parts of the message, not formatted.
     * @returns {string} The constructed body.
     */

  }, {
    key: 'buildBody',
    value: function buildBody(parts) {
      var messages = parts.map(this.options.format);
      return messages.join(messages.some(function (s) {
        return s.includes('\n');
      }) ? '\n' : ' ');
    }

    /**
     * Build the entire message.
     *
     * @param {number} level - Level number.
     * @param {*[]} parts - Parts of the message, not formatted.
     * @returns {string} The constructed message.
     */

  }, {
    key: 'buildMessage',
    value: function buildMessage(level, parts) {
      var header = this.buildHeader(level);
      var body = this.buildBody(parts);
      return '' + header + indent(body, (0, _stripAnsi2.default)(header).length) + '\n';
    }

    /**
     * Build a message and write it to the stream.
     *
     * @param {number} level - Level number.
     * @param {*[]} parts - Parts of the message, not formatted.
     * @returns {undefined} Nothing.
     */

  }, {
    key: 'write',
    value: function write(level, parts) {
      if (level > this.level) {
        return;
      }
      var stream = applyOption(this.options.stream, [level]);
      stream.write(this.buildMessage(level, parts));
    }
  }, {
    key: 'level',
    get: function get() {
      return this.options.levels.indexOf(this.options.level);
    }
  }]);

  return Log;
}();

/**
 * Utility function that will create a new instance of `Log` with options taken
 * from environment variables.
 *
 * Variables:
 * - `NODE_ENV`:
 *   - `'development'` (default): `displayTime` is disabled,
 *   - `'test'`: `displayTime` is disabled, `'level'` is set to `'critical'`,
 *   - `'production'`: Use default values for each option.
 * - `LOG`: Sets `level` value.
 * - `LOG_LEVEL`: Alias for `LOG`.
 * - `LOG_TIME`: When set to `true` or `1`, enables `displayTime`.
 * - `LOG_TIME_FORMAT`: Sets `displayTimeFormat` value.
 *
 * @param {Object} [options={}] - Additional options. Will take precedence over other ones.
 * @param {Object} [env=process.env || {}] - `env` object.
 * @param {string} [environment=env.NODE_ENV || 'development'] - Environment.
 * @returns {Log} A new instance of `Log`.
 */


function fromEnv() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var env = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : process.env || {};
  var environment = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : env.NODE_ENV || 'development';

  var mergedOptions = (0, _lodash2.default)(options, {
    level: env.LOG || env.LOG_LEVEL,
    displayTime: env.LOG_TIME && ['1', 'true'].indexOf(env.LOG_TIME) !== -1,
    displayTimeFormat: env.LOG_TIME_FORMAT
  }, {
    development: { displayTime: false },
    test: { level: 'critical', displayTime: false }
  }[environment]);
  return new Log(mergedOptions);
}